#!/usr/bin/perl

#----------------------------------------------------------------------
#-- 
#-- Modulname:
#-- ----------
#--
#--    log.pm
#--   
#--
#-- Description:
#-- ------------
#--
#--    The log.pm module is thought as a facility for logging messages.
#--    Currently only the function write_log_message will be exported.
#-- 
#--
#-- History:
#-- --------
#--    
#--    07|08|1999 - Initial startup
#--
#----------------------------------------------------------------------

package log;

use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(write_log_message write_log_msg_only_in_vmode);

#-- Enforce good manners
# use diagnostics;
# use strict;

#-- Import the LOCK_* constants
use Fcntl ':flock';


unshift(@INC, "/usr/local/falcon/lib/perl5");

#-- Include predefined error codes and constant values
use error_codes;
use const_values;

#-- Set the version for version checking
$VERSION = "07211999";

#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    write_log_message
#--   
#--
#-- Description:
#-- ------------
#--
#--    Writes an assembled message to the dedicated file.
#--    The actual path and file name is configured in const_values.pl.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       In a scalar context you get only a return code.
#--       In a list context you get the return code and an error text which
#--       is the corresponding system error string for errno.
#-- 
#--       The return codes are
#--       
#--       $success: Everything worked fine.
#--       $open_file_failed: Something went wrong on opening the file.
#--       $lock_file_failed: Something went wrong on locking the file.
#--       $close_file_failed: Something went wrong on closing the file.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    Writes a message to the dedicated file.
#--
#----------------------------------------------------------------------
sub write_log_message
{
   my ($return_code, $errno_txt, $log_message, $is_locked);

   $return_code = $success;
   $errno_txt = "";


   #-- Open the (pipe to the) logfile for appending messages.
   #-- Path and filename is stored in falcon_log_msg_pipe which is again one of our const_values.
   if (!open(LOGFILEPIPE, ">>$falcon_log_msg_pipe"))
   {
      $return_code = $open_file_failed;
      $errno_txt = "$!";

      return wantarray ? ($return_code, $errno_txt) : $return_code;
   }

   #-- Prohibit a simultaneously access to avoid a mixup in the logfile.
   $is_locked = lock_file_excl(LOGFILEPIPE);
   if ($is_locked)
   {
      $log_message = assemble_log_message($_[0], $_[1], $_[2], $_[3]);
      print(LOGFILEPIPE "$log_message");

      unlock_file(LOGFILEPIPE);
   }
   else
   {
      #-- Can't lock the file for exclusive access
      $log_message = assemble_log_message("File locking failed for falcon_log_msg_pipe!");
      print(LOGFILEPIPE "$log_message");

      $return_code = $lock_file_failed;
      $errno_txt = "$!";
   }

   #-- Even if the logfile could not be locked try to close it.
   if (!close(LOGFILEPIPE))
   {
      #-- Do not reject a previous error code
      $return_code = ($return_code == $success) ? $close_file_failed : $return_code;
      $errno_txt = ($errno_txt eq "") ? "$!" : $errno_txt;
   }

   wantarray ? ($return_code, $errno_txt) : $return_code;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    write_log_msg_only_in_vmode
#--   
#--
#-- Description:
#-- ------------
#--
#--    This is a wrapper for write_log_message with one additional functionality
#--    which is testing the last parameter with the meaning of 'verbose_mode_is_on'.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--       $verbose_mode_is_on: True - log the message
#--                            False - do not log the message
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       See write_log_message.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    If verbose mode is active then write a message to the dedicated file.
#--
#----------------------------------------------------------------------
sub write_log_msg_only_in_vmode
{
   my ($return_code, $errno_txt);

   $return_code = $success;
   $errno_txt = "";

   if ($_[4])
   {
      ($return_code, $errno_txt) = write_log_message($_[0], $_[1], $_[2], $_[3]);
   }

   wantarray ? ($return_code, $errno_txt) : $return_code;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    assemble_log_message
#--   
#--
#-- Description:
#-- ------------
#--
#--    Assembles a complete log message consisting of:
#--       <date> <time> <component> <(active_configuration)> <[PID]> <error_class>,
#--       <severity_code>: <message_text>
#--
#--    The subroutine caller has to provide the parts active_configuration,
#--    error_class, severity_code and the message_text.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       $log_message: The finished version of a log message.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    None
#--
#----------------------------------------------------------------------
sub assemble_log_message
{
   #-- Parts of the log message
   my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
   my ($component, $active_configuration, $severity_code, $error_class, $message_text);

   $active_configuration = shift;
   $error_class = shift; 
   $severity_code = shift;
   $message_text = shift;  

   #-- Get the timestamp
   ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

   #-- The executing program will be considered as a proxy component
   $component = $0;
   $component =~ s/.*\///;

   #-- Assemble the parts and build the final log message
   $log_message = sprintf("%02d|%02d|%4d %02d:%02d:%02d %s (%s) [$$] %s, %s: %s\n", $mon + 1, $mday, $year + 1900, $hour, $min, $sec, $component, $active_configuration, $error_class, $severity_code, $message_text);
   

   $log_message;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    lock_file_excl
#--   
#--
#-- Description:
#-- ------------
#--
#--    Lock the delivered file for exclusive access.
#--    In the current state this subroutine is only slightly more than a wrapper.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $filename: Name of the file that should be locked.
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       $is_locked: TRUE means success, FALSE on failure.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    A locked file.
#--
#----------------------------------------------------------------------
sub lock_file_excl
{
   my ($filename, $is_locked);
   $filename = shift;

   #-- Lock the file for exclusive access.
   #-- 'flock' produces a fatal error if used on a machine that doesn't implement
   #-- it, this is the reason for the use of eval. See perlfunc for details.
   $is_locked = eval(flock($filename, LOCK_EX));

   #-- Just in case ... 
   #-- Make sure that the appending action takes place at the end of the file.
   seek($filename, 0, 2);     

   $is_locked;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    unlock_file
#--   
#--
#-- Description:
#-- ------------
#--
#--    Release the locked file.  
#--    In the current state this subroutine is only a wrapper for flock.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $filename: Name of the locked file.
#-- 
#--    In-Out:
#--       None
#--
#--    Out:
#--       None
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    A formerly locked file will be released.
#--
#----------------------------------------------------------------------
sub unlock_file
{
   my ($filename);
   $filename = shift;

   #-- Release the lock
   flock($filename, LOCK_UN);
}



1;
