#!/usr/bin/perl

#----------------------------------------------------------------------
#-- 
#-- Modulename:
#-- -----------
#--
#--    mq_log.pm
#--   
#--
#-- Description:
#-- ------------
#--
#--    The mq_log.pm module is thought as a facility for logging messages.
#--    Currently the functions
#--       -assemble_log_message
#--       -write_log_message
#--       -write_log_msg_only_in_vmode
#--    will be exported.
#--
#--
#-- History:
#-- --------
#--    
#--    07|27|1999 - Initial startup
#--
#----------------------------------------------------------------------

package mq_log;

use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw(assemble_log_message write_log_message write_log_msg_only_in_vmode);

#-- Include the Falcon lib path into the perl search path.
use lib "/usr/local/falcon/lib/perl5";

#-- Enforce good manners
#use diagnostics;
use strict;

#-- Include predefined error codes and constant values
use error_codes;
use const_values;

#-- Set the version for version checking
my $version = "07271999";

#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    write_log_message
#--   
#--
#-- Description:
#-- ------------
#--
#--    Writes an assembled message to the dedicated file.
#--    The actual path and file name is configured in const_values.pl.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       In a scalar context you get only a return code.
#--       In a list context you get the return code and an error text which
#--       is the corresponding system error string for errno.
#-- 
#--       The return codes are
#--       
#--       $success: Everything worked fine.
#--       $open_file_failed: Something went wrong on opening the file.
#--       $lock_file_failed: Something went wrong on locking the file.
#--       $close_file_failed: Something went wrong on closing the file.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    Writes a message to the dedicated file.
#--
#----------------------------------------------------------------------
sub write_log_message
{
   my ($return_code, $errno_txt, $log_message, $is_locked, $message_queue_id, $has_msg_sent);

   $return_code = $success;
   $errno_txt = "";
   $has_msg_sent = $false;

   
   $log_message = assemble_log_message($_[0], $_[1], $_[2], $_[3]);

   #-- Insert the assembled message into the log message queue.
   $message_queue_id = msgget($log_message_queue_key, 0);
   if (defined $message_queue_id)
   {
      $has_msg_sent = msgsnd($message_queue_id, pack("l", 1) . $log_message, 0);
      if (!$has_msg_sent)
      {
         $return_code = $send_mq_msg_failed;
         $errno_txt = "$!";
      } 
   }
   else
   {
      $return_code = $get_mq_id_failed;
      $errno_txt = "$!";
   }

   #-- In the case that the log message can not be inserted into the message queue
   #-- it will be forwarded to 'standard error'. 
   if (!$has_msg_sent)
   {
      print(STDERR "$log_message");
   }

   wantarray ? ($return_code, $errno_txt) : $return_code;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    write_log_msg_only_in_vmode
#--   
#--
#-- Description:
#-- ------------
#--
#--    This is a wrapper for write_log_message with one additional functionality
#--    which is testing the last parameter with the meaning of 'verbose_mode_is_on'.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--       $verbose_mode_is_on: True - log the message
#--                            False - do not log the message
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       See write_log_message.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    If verbose mode is active then write a message to the dedicated file.
#--
#----------------------------------------------------------------------
sub write_log_msg_only_in_vmode
{
   my ($return_code, $errno_txt);

   $return_code = $success;
   $errno_txt = "";

   if ($_[4])
   {
      ($return_code, $errno_txt) = write_log_message($_[0], $_[1], $_[2], $_[3]);
   }

   wantarray ? ($return_code, $errno_txt) : $return_code;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    assemble_log_message
#--   
#--
#-- Description:
#-- ------------
#--
#--    Assembles a complete log message consisting of:
#--       <date> <time> <component> <(active_configuration)> <[PID]> <error_class>,
#--       <severity_code>: <message_text>
#--
#--    The subroutine caller has to provide the parts active_configuration,
#--    error_class, severity_code and the message_text.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $active_configuration: Section name from config file.
#--
#--       $error_class: This should be a word which classifies the error.
#--                     You can build your own scheme or just adopt our's.
#--                     For details on error classes see documentation. 
#--
#--       $severity_code: This should be a short (one word) description of the
#--                       event severity. As mentioned above you can build
#--                       your own scheme.
#--                       For details on severity codes see documentation.
#--
#--       $message_text: A (hopefully high grade) message text
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       $log_message: The finished version of a log message.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    None
#--
#----------------------------------------------------------------------
sub assemble_log_message
{
   #-- Parts of the log message
   my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
   my ($component, $log_message);
   my ($active_configuration, $error_class, $severity_code, $message_text) = @_;

   #-- Get the timestamp
   ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

   #-- The executing program will be considered as a proxy component.
   $component = $0;
   $component =~ s/.*\///;

   #-- Assemble the parts and build the final log message.
   $log_message = sprintf("%02d|%02d|%4d %02d:%02d:%02d %s (%s) [$$] %s, %s: %s", $mon + 1, $mday, $year + 1900, $hour, $min, $sec, $component, $active_configuration, $error_class, $severity_code, $message_text);
   

   $log_message;
}



1;
