#!/usr/bin/perl
#----------------------------------------------------------------------
#-- 
#-- Modulename:
#-- -----------
#--
#--    readcfg.pm
#--   
#--
#-- Description:
#-- ------------
#--
#--    This file includes routines for parsing a configurationfile and
#--    giving back the values to the mainprogram
#-- 
#--
#-- History:
#-- --------
#--
#--    07|08|1999 - Initial Startup
#--    12|01|1999 - readcfg now is a package
#--
#----------------------------------------------------------------------

package readcfg;

use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw(readcfg value_in_array);


use lib "/usr/local/falcon/lib/perl5";
use const_values;
use mq_log;

#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    readcfg
#--   
#--
#-- Description:
#-- ------------
#--
#--    Reads the configuration-file which is defined in the global
#--    variable $cfgfile in const_values.pl. Since the config-
#--    file is divided into sections, the calling program must
#--    give the name of the section to use.
#--    A section called global-settings is generally parsed to
#--    give the possibility of creating something like a template.
#--    Directives from the global section are overwritten by those
#--    from the section which name is given by the calling program.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       scalar section-name
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       hash config(keys|values)
#--
#--
#-- 
#-- Side-effects:
#-- -------------
#--
#--    If an unknown keyword is found in the configfile, the program
#--    aborts. Thus typos can prevented.
#--
#----------------------------------------------------------------------
sub readcfg
{
   my $local_section_name = shift;

   #-- These are for remembering if we're in a section or not
   my $in_global_section  = 0;
   my $in_local_section   = 0;

   #-- For the lines in the global-section
   my @global_configlines = ();
   #-- For the lines in the local-section
   my @local_configlines  = ();

   #-- Open, read and close configfile
   if (!open (CFGFILE,"<$cfgfile"))
   {
      write_log_message($options{"s"}, "Config", "Error", "Could not find configurationfile!");
      exit;
   }
   my @cfglines           = <CFGFILE>;
   close(CFGFILE);

   #-- Check if the specified section exists at all
   my $section_found = 0;
   foreach my $line (@cfglines)
   {
      if ($line =~ /^\s*section\s*$local_section_name\s*\:/) {
         $section_found = 1;
      }
   }
   #-- Exit if the specified Section is not found
   if (!$section_found)
   {
      write_log_message($options{"s"}, "Config", "Error", "Section \"$local_section_name\" does not exist in \"$cfgfile\".");
      exit;
   }

   #-- For every line in the configfile do
   foreach my $line (@cfglines)
   {
      #-- Cut the newline
      chomp $line;
      #-- If the line has a comment cut it off
      if ($line =~ /#/)
      {
         my @HELP = split /#/,$line;
         $line = $HELP[0];
      }
      #-- Mark if the global section has begun
      if ($line =~ /$global_section_name/)
      {
         $in_global_section = 1;
         next;
      }
      #-- Mark if the local section has begun
      if ($line =~ /$local_section_name/)
      {
         $in_local_section = 1;
         next;
      }
      #-- Mark if a section has ended
      if ($line =~ /\}/)
      {
         $in_global_section = 0;
         $in_local_section  = 0;
      }
      #-- Ignore the line if it contains no character or number
      if ($line !~ /\w/)
      {
         next;
      }

      #-- Change all tabs into spaces
      $line =~ s/\t/ /g;
      #-- Erase all multiple spaces
      while ($line =~ s/\s\s/ /g){};
      #-- Erase the space at the beginning of the line (if exists)
      $line =~ s/^\s//;
      #-- Erase the space at the end of the line (if exists)
      $line =~ s/\s$//;

      #-- Remember this line if it's relevant
      if ($in_global_section)
      {
         push @global_configlines, $line;
      }
      if ($in_local_section)
      {
         push @local_configlines, $line;
      }
   }

   my %config_entries = ();

   #-- For all lines in the global section do...
   #--  - split the keyword/value pairs
   #--  - uppercase the keyword
   #--  - check the keyword and exit if its invalid
   #--  - else put the key/value-pair onto the hash
   foreach my $line (@global_configlines)
   {
      my $key = uc((split " ",$line)[0]);
      if (!(value_in_array($key,@valid_config_keywords)))
      {
         write_log_message($options{"s"}, "Config", "Error", "$key is not a valid configuration keyword (Section: $global_section_name)!");
         exit;
      }
      $line =~ s/^$key //i;
      $config_entries{$key} = $line;
   }

   #-- For all lines in the local section do...
   #-- the same as for those in the global section.
   #-- Directives with the same keywords overwrite those
   #-- from the global section
   foreach my $line (@local_configlines)
   {
      my $key = uc((split " ",$line)[0]);
      if (!(value_in_array($key,@valid_config_keywords)))
      {
         write_log_message($options{"s"}, "Config", "Error", "$key is not a valid configuration keyword (Section: $local_section_name)!");
         exit;
      }
      $line =~ s/^$key //i;
      $config_entries{$key} = $line;
   }

   return %config_entries;
}

#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    value_in_array
#--   
#--
#-- Description:
#-- ------------
#--
#--    Checks if a specified array contains a specified value.
#--    Returns 1 if so, 0 otherwise.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       value, array
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       0 or 1
#--
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    None
#--
#----------------------------------------------------------------------
sub value_in_array
{
   my @array = @_;
   my $count = 1;
   while ($array[$count])
   {
      if ($array[0] eq $array[$count])
      {
         return 1;
      }
      $count++;
   }
   return 0;
}

1;