#!/usr/bin/perl

#----------------------------------------------------------------------
#--
#-- Programname:
#-- ------------
#--
#--    falconlogd.pl
#--
#--
#-- Description:
#-- ------------
#--
#--    This deamon is responsible for reading entries from the log
#--    message queue and for storing them to the falcon.log file.
#--
#--
#-- Arguments:
#-- ----------
#--
#--    In:
#--    In-Out:
#--    Out:
#--
#--
#-- History:
#-- --------
#--
#--    07|27|1999 Initial startup
#--    11|30|1999 Now writes its PID to $pid_file
#--
#----------------------------------------------------------------------

#-- Enforce good manners
use diagnostics;
use strict;

#-- Include (in borg slang assimilate) the Falcon lib path into the perl search path.
use lib "/usr/local/falcon/lib/perl5";

#-- Include predefined error codes and constant values.
#-- These modules are located in '/usr/local/falcon/lib/perl5' therefore
#-- the perl interpreter must know about this 'new' path.
use error_codes;
use const_values;

#-- Include functions from the logging module.
use mq_log;

# For writing the pid to a file
use pid;

#-- Include object methods for filehandles.
use IO::File;

#-- Include the 'deamon mode' and a convenient 'get option' functionality.
use Proc::Daemon;
use Getopt::Std;

#-- Make the variables known to the world of perl.
my ($version, $programname, $verbose_mode_is_on, $daemon_mode_is_on, $message_queue_id, $message_and_type, $message, $log_message, $type, %options, $logfileh);


#-- Set the version for version checking.
$version = "08021999";


#----------------------------------------------------------------------
#-- Evaluate the commandline arguments.
#----------------------------------------------------------------------
#-- Parse the commandline arguments ...
getopts("?drvV",\%options);

#-- ... and assign the values to their respective variables.
if (defined $options{"?"})
{
   $programname = $0;
   $programname =~ s/.*\///;
   print "Usage: $programname [-d] [-v]\n";
   exit;
}

if (defined $options{"V"})
{
   print("Falcon Log Daemon - Version $version\n");
   exit;
}

if (defined $options{"v"})
{
   $verbose_mode_is_on = $true;
}
else
{
   $verbose_mode_is_on = $false;
}

if (defined $options{"d"})
{
   $daemon_mode_is_on = $true;
}
else
{
   $daemon_mode_is_on = $false;
}


#-- Get the message queue id for further operations on the 'log' message queue.
$message_queue_id = msgget($log_message_queue_key, $message_queue_create | $message_queue_mode);

#-- You can remove a message queue by issuing the command 'ipcrm msg id', where id
#-- is the id of the message queue which is returned by 'ipcs -q'.
#-- However, for your convinient we provide this remove option.
if (defined $options{"r"})
{
   my ($ipc_rmid, $queue_removed);


   $ipc_rmid = 0;

   print("Removing message queue ");
   $queue_removed = msgctl($message_queue_id, $ipc_rmid, 0); 
   if (defined $queue_removed)
   {
      print("completed successfully!\n");
   }
   else
   {
      print("failed!\n");
   }

   exit;
}


#-- Configure handling of the TERM and INT signal.
$SIG{'TERM'} = 'terminate_logd';
$SIG{'INT'} = 'terminate_logd';


#----------------------------------------------------------------------
#-- Become a daemon (option -d) if desired.
#----------------------------------------------------------------------
if ($daemon_mode_is_on)
{
   #-- The Proc::Daemon::Init closes (among other things) all open file descriptors,
   #-- therefore the file opening stuff follows after this initialization.
   #-- For debugging purposes just start in non daemon mode!
   Proc::Daemon::Init;
}
write_pid();

#-- Open the logfile in append mode.
#-- Path and filename is stored in $logfile which is again one of our const_values.
$logfileh = new IO::File "$logfile", O_CREAT|O_WRONLY|O_APPEND;

if (defined $logfileh)
{
   $logfileh->autoflush(1);
}
else
{
   print(STDERR "Error: Can't open file $logfile! $!\n");
   exit;
}


#-- Assemble and perpetuate the start-up message.
$log_message = assemble_log_message("Log", "Proxy", "Info", "Logging facility started.");
safe_log_message($logfileh, $log_message);
if (!$daemon_mode_is_on)
{
   print("$log_message\n");
}


#----------------------------------------------------------------------
#-- Receive messages bit by bit from the queue and store them.
#----------------------------------------------------------------------
$message_and_type = 0;
while (msgrcv($message_queue_id, $message_and_type, $max_message_length, 0, 0))
{
   #-- Get rid of the message type.
   ($type, $message) = unpack "lA$max_message_length", $message_and_type;

   if (!$daemon_mode_is_on)
   {
      print("$message\n");
   }

   safe_log_message($logfileh, $message);
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    safe_log_message
#--
#--   
#-- Description:
#-- ------------
#--
#--    Perpetuate the delivered message in a logfile.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    In:
#--       $logfile: Name of the target file.
#--       $log_message: The log message.
#--
#--    In-Out:
#--       None
#--
#--    Out:
#--       In a scalar context you get only a return code.
#--       In a list context you get the return code and an error text which
#--       is the corresponding system error string for errno.
#-- 
#--       The return codes are the ones delivered by function print.
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    None
#--
#----------------------------------------------------------------------
sub safe_log_message
{
   my ($return_code, $errno_txt);
   my ($logfile, $log_message) = @_;


   $return_code = $success;
   $errno_txt = "";

   $return_code = print $logfile "$log_message\n";
   $errno_txt = "$!";

   wantarray ? ($return_code, $errno_txt) : $return_code;
}



#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    terminate_logd
#--
#--   
#-- Description:
#-- ------------
#--
#--    On receiving a SIGTERM (default if no signal is specified at kill
#--    command) or SIGINT signal (Ctrl-C) close the logfile and leave
#--    the program.
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    None
#--
#----------------------------------------------------------------------
sub terminate_logd
{
   my ($log_message);


   #-- Assemble and perpetuate the end message.
   $log_message = assemble_log_message("Log", "Proxy", "Info", "@_ signal received. Program terminated gracefully.");
   safe_log_message($logfileh, $log_message);
   
   if (!$daemon_mode_is_on)
   {
      print("$log_message\n");
   }

   $logfileh->close;
   remove_pid();
   exit;
}
