#!/usr/bin/perl

#----------------------------------------------------------------------
#-- 
#-- Programname:
#-- ------------
#--   
#--    tcp-plug.pl
#--
#--   
#-- Description:
#-- ------------
#--   
#--    A generic tcp-plug
#--   
#-- 
#-- Arguments:
#-- ----------
#--   
#--    In:
#--    In-Out:
#--    Out:
#--
#--
#-- History:
#-- --------
#--
#--    07|08|1999 Initial startup
#--    12|01|1999 writes pid to $pid_file
#--
#----------------------------------------------------------------------

#-- Include the Falcon lib path into the perl search path.
use lib "/usr/local/falcon/lib/perl5";

#-- Include predefined error codes and constant values.
#-- These modules are located in '/usr/local/falcon/lib/perl5' therefore
#-- the perl interpreter must know about this 'new' path.
use error_codes;
use const_values;

#-- Include functions from the logging module.
use mq_log;
use pid;
use IO::Socket;
use Proc::Daemon;
use Getopt::Std;

use readcfg;

#-- Parse the commandline arguments
getopts("?vVs:f:",\%options);

if (defined $options{"?"})
{
   $programname = $0;
   $programname =~ s/.*\///;
   print "Usage: $programname [-f Filename] [-v] -s SectionName\n";
   exit;
}

if (defined $options{"V"})
{
   print "TCP-Plug - Version 09071999\n";
   exit;
}

if (defined $options{"v"})
{
   $verbose_mode_is_on = $true;
}
else
{
   $verbose_mode_is_on = $false;
}

if (defined $options{"f"})
{
   $cfgfile = $options{"f"}
}

if (!defined $options{"s"})
{
   print "You have to give a section-name for configuration.\n";
   exit;
}


#-- Read the configfile
%config_values = readcfg($options{"s"});


#----------------------------------------------------------------------
#-- Network settings
#----------------------------------------------------------------------

#-- Port to listen on
$port = $config_values{"LISTEN-PORT"};

#-- Interface on which to listen on
#-- Clients will connect to this on port $port
$local_interface_intern  = $config_values{"LISTEN-INTERFACE"};

#-- Address of the server
$remote_interface        = $config_values{"TARGET-HOST"};

#-- Port to connect the server
$remote_port             = $config_values{"TARGET-PORT"};


#----------------------------------------------------------------------
#-- Security settings
#----------------------------------------------------------------------

#-- The directory for changeroot
$newroot = $config_values{"CHROOT-PATH"};

#-- The user- and group-id to run as
$euid = getpwnam($config_values{"RUN-WITH-UID"});
$egid = getgrnam($config_values{"RUN-WITH-GID"});

if (!($euid && $egid))
{
   print("\nError: Couldnt find user- or group-id in /etc/passwd or /etc/group.\n\n");
   exit;
}

#----------------------------------------------------------------------
#-- Become a daemon
#----------------------------------------------------------------------
#-- Comment out the next line for debugging purposes...
Proc::Daemon::Init;
#fork && exit;
write_pid();

#----------------------------------------------------------------------
#-- 
#-- Subroutinename:
#-- ---------------
#--
#--    terminate_clean
#--
#--   
#-- Description:
#-- ------------
#--
#--    A process enters this method if he gets an USR1-Signal
#--
#-- 
#-- Arguments:
#-- ----------
#--
#--    None
#--
#--
#-- Side-effects:
#-- -------------
#--
#--    The process entering this routine terminates.
#--
#----------------------------------------------------------------------
sub terminate_clean
{
   remove_pid();
   sleep 2;
   write_log_msg_only_in_vmode($options{"s"}, "Proxy", "Info", "USR1-Signal received.", $verbose_mode_is_on);

   $local_clientside->close;
   $local_serverside->close;
   exit;
}



   #-- Configure the handling of signals
   $SIG{'CHLD'} = sub { wait };
   $SIG{'USR1'} = 'terminate_clean';

#-- Start listening. Listening on ports up to 1024 is restricted to the superuser!
$listener_socket = IO::Socket::INET->new(Proto     => "tcp",
                                         Type      => SOCK_STREAM,
                                         LocalPort => $port,
                                         LocalAddr => $local_interface_intern,
                                         Listen    => 1,
                                         Reuse     => 1
                                        );

#-- Change the root-directory to a new -virtual- one.
#-- Only the superuser can perform this call!
$switched_to_new_root = chroot $newroot;

#-- Change the effective user- and group-id.
#-- Don't log any messages (using write_log_message) before the new
#-- user- and group-id are activated because this will affect (at creation time)
#-- the owner of the logfile!
$> = $euid;
$) = $egid;


#----------------------------------------------------------------------
#-- For the following operations, minor access rights must be sufficient.
#----------------------------------------------------------------------

#-- It is probably a little bit late for logging this but it is intended.
#-- For details see comment above...
if ($verbose_mode_is_on)
{
  write_log_message($options{"s"}, "Proxy", "Info", "Started in verbose mode.");
}

if ($switched_to_new_root)
{
   write_log_msg_only_in_vmode($options{"s"}, "Proxy", "Info", "The new root directory is $newroot.", $verbose_mode_is_on);
}
else
{
   write_log_message($options{"s"}, "System", "Error", "Can't make $newroot to the new root directory: $@.");
   exit;
}

#-- Error handling for the listener socket
if (!$listener_socket)
{
   write_log_message($options{"s"}, "System", "Error", "Can't open listener socket: $@.");
   exit;
}


#----------------------------------------------------------------------
#-- Forever (till someone sends a SIGTERM or something similar)
#-- do the Proxying-Stuff
#----------------------------------------------------------------------
do
{
   #-- Wow! Someone talks to us!
   $local_clientside = $listener_socket->accept;
   print "Connection Accepted";
   if ($local_clientside)
   {
      #-- Incoming socket connection accepted
      write_log_msg_only_in_vmode($options{"s"}, "Proxy", "Info", "Incoming socket connection accepted (local address: $local_interface_intern, local port: $port).", $verbose_mode_is_on);
   }
   else
   {
      #-- Error occured
      write_log_message($options{"s"}, "System", "Error", "The incoming socket connection (local address: $local_interface_intern, local port: $port) reported an error: $!.");
   }

   #-- Turn buffering off
   $local_clientside->autoflush(1);
   #----------------------------------------------------------------------
   #-- Split the program into two processes, identical twins.
   #-- One process does the handling of the client, the other one
   #-- listens on the socket again to be ready for handling new clients
   #----------------------------------------------------------------------
   if (!defined($childpid = fork()))
   {
      write_log_message($options{"s"}, "System", "Error", "Can't fork: $!.");
      exit;
   }

   #-- If we are the child-process then we have to handle the connection
   if (!$childpid)
   {
      #-- Make a Connection to the specified server
      $local_serverside = IO::Socket::INET->new(Proto     => "tcp",
                                                Type      => SOCK_STREAM,
                                                PeerPort  => $remote_port,
                                                PeerAddr  => $remote_interface
                                               );
      if (!$local_serverside)
      {
         write_log_message($options{"s"}, "System", "Error", "Can't open socket to Server: $@.");
         exit;
      }

      #-- Turn buffering off
      $local_serverside->autoflush(1);

      #----------------------------------------------------------------------
      #-- Split another time. One process transfers the data from the server
      #-- to the client, its child transfers data from the client to the server.
      #-- This seems to be the easiest way of doing some bidirectional
      #-- asynchronous communication over sockets.
      #----------------------------------------------------------------------
      if (!defined($kidpid = fork()))
      {
         write_log_message($options{"s"}, "System", "Error", "Can't fork: $!.");
         exit;
      }

      #-- If were the father send data from server to client.
      if ($kidpid)
      {
         #-- As long as the server wants to talk to us, we send all the stuff he
         #-- says to the client.
         while (defined ($line = <$local_serverside>))
         {
            print $local_clientside $line;
         }

         #-- Oh, the server went away. Now we close the connection to him and the client
         #-- and tell our child-process (who is transfering the data from the client to
         #-- the server) to die. Then we also die (thats live!).
         $local_serverside->close;
         $local_clientside->close;

         #-- send SIGTERM to child
         kill("USR1", $kidpid);
         exit;
      }
      #-- Else send data from client to server.
      else
      {
         #-- As long as the client wants to talk to us, we send all the stuff he says
         #-- to the server.
         while (defined ($line = <$local_clientside>))
         {
            print $local_serverside $line;
         }

         #-- It may also happen that the client goes away first.
         #-- In that case we close the connection to him and the server and tell our 
         #-- daddy-process (who is transfering the data from the server to
         #-- the client) to die. The exit seems not needed because if our father dies
         #-- he will send us a SIGTERM, too. But who knows what its good for?
         $local_clientside->close;
         $local_serverside->close;
         kill("USR1", getppid());
         exit;
      }
   }
   #-- If we're the father of all the processes get rid of the client and listen on
   #-- the Port again
   else
   {
#      $local_clientside->close;
   }
} while (1);